@@ -1,8 +1,5 @@
 #pragma once
 
-#ifndef FASTFETCH_INCLUDED
-#define FASTFETCH_INCLUDED
-
 #include "fastfetch_config.h"
 
 #include <stdint.h>
@@ -23,19 +20,27 @@
 #include "util/platform/FFPlatform.h"
 #include "util/unused.h"
 
-#include "options/modules.h"
 #include "options/logo.h"
 #include "options/display.h"
 #include "options/general.h"
-#include "options/library.h"
+
+#ifdef __has_builtin
+    #if __has_builtin(__is_array)
+        #define ARRAY_SIZE(x) ({ static_assert(__is_array(__typeof__(x)), "Must be an array"); (uint32_t) (sizeof(x) / sizeof(*(x))); })
+    #elif __has_builtin(__builtin_types_compatible_p)
+        #define ARRAY_SIZE(x) ({ static_assert(!__builtin_types_compatible_p(__typeof__(x), __typeof__(&*(x))), "Must not be a pointer"); (uint32_t) (sizeof(x) / sizeof(*(x))); })
+    #endif
+#endif
+#ifndef ARRAY_SIZE
+    #define ARRAY_SIZE(x) ((uint32_t) (sizeof(x) / sizeof(*(x))))
+#endif
+
 
 typedef struct FFconfig
 {
     FFOptionsLogo logo;
     FFOptionsDisplay display;
     FFOptionsGeneral general;
-    FFOptionsModules modules;
-    FFOptionsLibrary library;
 } FFconfig;
 
 typedef struct FFstate
@@ -43,11 +48,14 @@ typedef struct FFstate
     uint32_t logoWidth;
     uint32_t logoHeight;
     uint32_t keysHeight;
+    bool terminalLightTheme;
+    bool titleFqdn;
 
     FFPlatform platform;
     yyjson_doc* configDoc;
     yyjson_mut_doc* resultDoc;
     FFstrbuf genConfigPath;
+    bool fullConfig;
 } FFstate;
 
 typedef struct FFinstance
@@ -57,29 +65,3 @@ typedef struct FFinstance
 } FFinstance;
 extern FFinstance instance; // Defined in `common/init.c`
 extern FFModuleBaseInfo** ffModuleInfos[];
-
-//////////////////////
-// Init functions //
-//////////////////////
-
-//common/init.c
-void ffInitInstance();
-void ffStart();
-void ffFinish();
-void ffDestroyInstance();
-
-void ffListFeatures();
-
-////////////////////
-// Logo functions //
-////////////////////
-
-void ffLogoPrint();
-void ffLogoPrintRemaining();
-void ffLogoPrintLine();
-
-void ffLogoBuiltinPrint();
-void ffLogoBuiltinList();
-void ffLogoBuiltinListAutocompletion();
-
-#endif
