@@ -1,7 +1,7 @@
 #include "common/printing.h"
 #include "common/jsonconfig.h"
 #include "common/percent.h"
-#include "common/parsing.h"
+#include "common/duration.h"
 #include "common/temps.h"
 #include "detection/battery/battery.h"
 #include "modules/battery/battery.h"
@@ -27,17 +27,6 @@ static void printBattery(FFBatteryOptions* options, FFBatteryResult* result, uin
         }));
     }
 
-
-    uint32_t timeRemaining = result->timeRemaining < 0 ? 0 : (uint32_t) result->timeRemaining;
-
-    uint32_t seconds = timeRemaining % 60;
-    timeRemaining /= 60;
-    uint32_t minutes = timeRemaining % 60;
-    timeRemaining /= 60;
-    uint32_t hours = timeRemaining % 24;
-    timeRemaining /= 24;
-    uint32_t days = timeRemaining;
-
     FFPercentageTypeFlags percentType = options->percent.type == 0 ? instance.config.display.percentType : options->percent.type;
 
     if(options->moduleArgs.outputFormat.length == 0)
@@ -70,7 +59,7 @@ static void printBattery(FFBatteryOptions* options, FFBatteryResult* result, uin
                 if(str.length > 0)
                     ffStrbufAppendS(&str, " (");
 
-                ffParseDuration(days, hours, minutes, seconds, &str);
+                ffDurationAppendNum((uint32_t) result->timeRemaining, &str);
                 ffStrbufAppendS(&str, " remaining)");
             }
         }
@@ -83,7 +72,7 @@ static void printBattery(FFBatteryOptions* options, FFBatteryResult* result, uin
                 ffStrbufAppend(&str, &result->status);
         }
 
-        if(result->temperature == result->temperature) //FF_BATTERY_TEMP_UNSET
+        if(result->temperature != FF_BATTERY_TEMP_UNSET)
         {
             if(str.length > 0)
                 ffStrbufAppendS(&str, " - ");
@@ -95,6 +84,15 @@ static void printBattery(FFBatteryOptions* options, FFBatteryResult* result, uin
     }
     else
     {
+        uint32_t timeRemaining = result->timeRemaining < 0 ? 0 : (uint32_t) result->timeRemaining;
+        uint32_t seconds = timeRemaining % 60;
+        timeRemaining /= 60;
+        uint32_t minutes = timeRemaining % 60;
+        timeRemaining /= 60;
+        uint32_t hours = timeRemaining % 24;
+        timeRemaining /= 24;
+        uint32_t days = timeRemaining;
+
         FF_STRBUF_AUTO_DESTROY capacityNum = ffStrbufCreate();
         if(percentType & FF_PERCENTAGE_TYPE_NUM_BIT)
             ffPercentAppendNum(&capacityNum, result->capacity, options->percent, false, &options->moduleArgs);
@@ -103,6 +101,9 @@ static void printBattery(FFBatteryOptions* options, FFBatteryResult* result, uin
             ffPercentAppendBar(&capacityBar, result->capacity, options->percent, &options->moduleArgs);
         FF_STRBUF_AUTO_DESTROY tempStr = ffStrbufCreate();
         ffTempsAppendNum(result->temperature, &tempStr, options->tempConfig, &options->moduleArgs);
+        FF_STRBUF_AUTO_DESTROY timeStr = ffStrbufCreate();
+        if (result->timeRemaining > 0)
+            ffDurationAppendNum((uint32_t) result->timeRemaining, &timeStr);
 
         FF_PRINT_FORMAT_CHECKED(key.chars, 0, &options->moduleArgs, FF_PRINT_TYPE_NO_CUSTOM_KEY, ((FFformatarg[]) {
             FF_FORMAT_ARG(result->manufacturer, "manufacturer"),
@@ -119,6 +120,7 @@ static void printBattery(FFBatteryOptions* options, FFBatteryResult* result, uin
             FF_FORMAT_ARG(hours, "time-hours"),
             FF_FORMAT_ARG(minutes, "time-minutes"),
             FF_FORMAT_ARG(seconds, "time-seconds"),
+            FF_FORMAT_ARG(timeStr, "time-formatted"),
         }));
     }
 }
@@ -157,45 +159,17 @@ void ffPrintBattery(FFBatteryOptions* options)
     }
 }
 
-bool ffParseBatteryCommandOptions(FFBatteryOptions* options, const char* key, const char* value)
-{
-    const char* subKey = ffOptionTestPrefix(key, FF_BATTERY_MODULE_NAME);
-    if (!subKey) return false;
-    if (ffOptionParseModuleArgs(key, subKey, value, &options->moduleArgs))
-        return true;
-
-    if (ffTempsParseCommandOptions(key, subKey, value, &options->temp, &options->tempConfig))
-        return true;
-
-    #ifdef _WIN32
-        if (ffStrEqualsIgnCase(subKey, "use-setup-api"))
-        {
-            options->useSetupApi = ffOptionParseBoolean(value);
-            return true;
-        }
-    #endif
-
-    if (ffPercentParseCommandOptions(key, subKey, value, &options->percent))
-        return true;
-
-    return false;
-}
-
 void ffParseBatteryJsonObject(FFBatteryOptions* options, yyjson_val* module)
 {
-    yyjson_val *key_, *val;
+    yyjson_val *key, *val;
     size_t idx, max;
-    yyjson_obj_foreach(module, idx, max, key_, val)
+    yyjson_obj_foreach(module, idx, max, key, val)
     {
-        const char* key = yyjson_get_str(key_);
-        if(ffStrEqualsIgnCase(key, "type"))
-            continue;
-
         if (ffJsonConfigParseModuleArgs(key, val, &options->moduleArgs))
             continue;
 
         #ifdef _WIN32
-        if (ffStrEqualsIgnCase(key, "useSetupApi"))
+        if (unsafe_yyjson_equals_str(key, "useSetupApi"))
         {
             options->useSetupApi = yyjson_get_bool(val);
             continue;
@@ -208,25 +182,20 @@ void ffParseBatteryJsonObject(FFBatteryOptions* options, yyjson_val* module)
         if (ffPercentParseJsonObject(key, val, &options->percent))
             continue;
 
-        ffPrintError(FF_BATTERY_MODULE_NAME, 0, &options->moduleArgs, FF_PRINT_TYPE_DEFAULT, "Unknown JSON key %s", key);
+        ffPrintError(FF_BATTERY_MODULE_NAME, 0, &options->moduleArgs, FF_PRINT_TYPE_DEFAULT, "Unknown JSON key %s", unsafe_yyjson_get_str(key));
     }
 }
 
 void ffGenerateBatteryJsonConfig(FFBatteryOptions* options, yyjson_mut_doc* doc, yyjson_mut_val* module)
 {
-    __attribute__((__cleanup__(ffDestroyBatteryOptions))) FFBatteryOptions defaultOptions;
-    ffInitBatteryOptions(&defaultOptions);
-
-    ffJsonConfigGenerateModuleArgsConfig(doc, module, &defaultOptions.moduleArgs, &options->moduleArgs);
+    ffJsonConfigGenerateModuleArgsConfig(doc, module, &options->moduleArgs);
 
     #ifdef _WIN32
-    if (defaultOptions.useSetupApi != options->useSetupApi)
         yyjson_mut_obj_add_bool(doc, module, "useSetupApi", options->useSetupApi);
     #endif
 
-    ffTempsGenerateJsonConfig(doc, module, defaultOptions.temp, defaultOptions.tempConfig, options->temp, options->tempConfig);
-
-    ffPercentGenerateJsonConfig(doc, module, defaultOptions.percent, options->percent);
+    ffTempsGenerateJsonConfig(doc, module, options->temp, options->tempConfig);
+    ffPercentGenerateJsonConfig(doc, module, options->percent);
 }
 
 void ffGenerateBatteryJsonResult(FFBatteryOptions* options, yyjson_mut_doc* doc, yyjson_mut_val* module)
@@ -252,10 +221,15 @@ void ffGenerateBatteryJsonResult(FFBatteryOptions* options, yyjson_mut_doc* doc,
         yyjson_mut_obj_add_strbuf(doc, obj, "status", &battery->status);
         yyjson_mut_obj_add_strbuf(doc, obj, "technology", &battery->technology);
         yyjson_mut_obj_add_strbuf(doc, obj, "serial", &battery->serial);
-        yyjson_mut_obj_add_real(doc, obj, "temperature", battery->temperature);
+        if (battery->temperature != FF_BATTERY_TEMP_UNSET)
+            yyjson_mut_obj_add_real(doc, obj, "temperature", battery->temperature);
+        else
+            yyjson_mut_obj_add_null(doc, obj, "temperature");
         yyjson_mut_obj_add_uint(doc, obj, "cycleCount", battery->cycleCount);
         if (battery->timeRemaining > 0)
             yyjson_mut_obj_add_int(doc, obj, "timeRemaining", battery->timeRemaining);
+        else
+            yyjson_mut_obj_add_null(doc, obj, "timeRemaining");
     }
 
     FF_LIST_FOR_EACH(FFBatteryResult, battery, results)
@@ -269,10 +243,28 @@ void ffGenerateBatteryJsonResult(FFBatteryOptions* options, yyjson_mut_doc* doc,
     }
 }
 
-static FFModuleBaseInfo ffModuleInfo = {
+void ffInitBatteryOptions(FFBatteryOptions* options)
+{
+    ffOptionInitModuleArg(&options->moduleArgs, "");
+    options->temp = false;
+    options->tempConfig = (FFColorRangeConfig) { 60, 80 };
+    options->percent = (FFPercentageModuleConfig) { 50, 20, 0 };
+
+    #ifdef _WIN32
+        options->useSetupApi = false;
+    #endif
+}
+
+void ffDestroyBatteryOptions(FFBatteryOptions* options)
+{
+    ffOptionDestroyModuleArg(&options->moduleArgs);
+}
+
+FFModuleBaseInfo ffBatteryModuleInfo = {
     .name = FF_BATTERY_MODULE_NAME,
     .description = "Print battery capacity, status, etc",
-    .parseCommandOptions = (void*) ffParseBatteryCommandOptions,
+    .initOptions = (void*) ffInitBatteryOptions,
+    .destroyOptions = (void*) ffDestroyBatteryOptions,
     .parseJsonObject = (void*) ffParseBatteryJsonObject,
     .printModule = (void*) ffPrintBattery,
     .generateJsonResult = (void*) ffGenerateBatteryJsonResult,
@@ -292,23 +284,6 @@ static FFModuleBaseInfo ffModuleInfo = {
         {"Battery time remaining hours", "time-hours"},
         {"Battery time remaining minutes", "time-minutes"},
         {"Battery time remaining seconds", "time-seconds"},
+        {"Battery time remaining (formatted)", "time-formatted"},
     }))
 };
-
-void ffInitBatteryOptions(FFBatteryOptions* options)
-{
-    options->moduleInfo = ffModuleInfo;
-    ffOptionInitModuleArg(&options->moduleArgs, "");
-    options->temp = false;
-    options->tempConfig = (FFColorRangeConfig) { 60, 80 };
-    options->percent = (FFPercentageModuleConfig) { 50, 20, 0 };
-
-    #ifdef _WIN32
-        options->useSetupApi = false;
-    #endif
-}
-
-void ffDestroyBatteryOptions(FFBatteryOptions* options)
-{
-    ffOptionDestroyModuleArg(&options->moduleArgs);
-}
