@@ -5,8 +5,6 @@
 #include "modules/camera/camera.h"
 #include "util/stringUtils.h"
 
-#define FF_CAMERA_NUM_FORMAT_ARGS 6
-
 static void printDevice(FFCameraOptions* options, const FFCameraResult* device, uint8_t index)
 {
     if(options->moduleArgs.outputFormat.length == 0)
@@ -27,13 +25,13 @@ static void printDevice(FFCameraOptions* options, const FFCameraResult* device,
     }
     else
     {
-        FF_PRINT_FORMAT_CHECKED(FF_CAMERA_MODULE_NAME, index, &options->moduleArgs, FF_PRINT_TYPE_DEFAULT, FF_CAMERA_NUM_FORMAT_ARGS, (((FFformatarg[]) {
-            {FF_FORMAT_ARG_TYPE_STRBUF, &device->name, "name"},
-            {FF_FORMAT_ARG_TYPE_STRBUF, &device->vendor, "vendor"},
-            {FF_FORMAT_ARG_TYPE_STRBUF, &device->colorspace, "colorspace"},
-            {FF_FORMAT_ARG_TYPE_STRBUF, &device->id, "id"},
-            {FF_FORMAT_ARG_TYPE_UINT, &device->width, "width"},
-            {FF_FORMAT_ARG_TYPE_UINT, &device->height, "height"},
+        FF_PRINT_FORMAT_CHECKED(FF_CAMERA_MODULE_NAME, index, &options->moduleArgs, FF_PRINT_TYPE_DEFAULT, (((FFformatarg[]) {
+            FF_FORMAT_ARG(device->name, "name"),
+            FF_FORMAT_ARG(device->vendor, "vendor"),
+            FF_FORMAT_ARG(device->colorspace, "colorspace"),
+            FF_FORMAT_ARG(device->id, "id"),
+            FF_FORMAT_ARG(device->width, "width"),
+            FF_FORMAT_ARG(device->height, "height"),
         })));
     }
 }
@@ -69,39 +67,22 @@ void ffPrintCamera(FFCameraOptions* options)
     }
 }
 
-bool ffParseCameraCommandOptions(FFCameraOptions* options, const char* key, const char* value)
-{
-    const char* subKey = ffOptionTestPrefix(key, FF_CAMERA_MODULE_NAME);
-    if (!subKey) return false;
-    if (ffOptionParseModuleArgs(key, subKey, value, &options->moduleArgs))
-        return true;
-
-    return false;
-}
-
 void ffParseCameraJsonObject(FFCameraOptions* options, yyjson_val* module)
 {
-    yyjson_val *key_, *val;
+    yyjson_val *key, *val;
     size_t idx, max;
-    yyjson_obj_foreach(module, idx, max, key_, val)
+    yyjson_obj_foreach(module, idx, max, key, val)
     {
-        const char* key = yyjson_get_str(key_);
-        if(ffStrEqualsIgnCase(key, "type"))
-            continue;
-
         if (ffJsonConfigParseModuleArgs(key, val, &options->moduleArgs))
             continue;
 
-        ffPrintError(FF_CAMERA_MODULE_NAME, 0, &options->moduleArgs, FF_PRINT_TYPE_DEFAULT, "Unknown JSON key %s", key);
+        ffPrintError(FF_CAMERA_MODULE_NAME, 0, &options->moduleArgs, FF_PRINT_TYPE_DEFAULT, "Unknown JSON key %s", unsafe_yyjson_get_str(key));
     }
 }
 
 void ffGenerateCameraJsonConfig(FFCameraOptions* options, yyjson_mut_doc* doc, yyjson_mut_val* module)
 {
-    __attribute__((__cleanup__(ffDestroyCameraOptions))) FFCameraOptions defaultOptions;
-    ffInitCameraOptions(&defaultOptions);
-
-    ffJsonConfigGenerateModuleArgsConfig(doc, module, &defaultOptions.moduleArgs, &options->moduleArgs);
+    ffJsonConfigGenerateModuleArgsConfig(doc, module, &options->moduleArgs);
 }
 
 void ffGenerateCameraJsonResult(FF_MAYBE_UNUSED FFCameraOptions* options, yyjson_mut_doc* doc, yyjson_mut_val* module)
@@ -136,35 +117,31 @@ void ffGenerateCameraJsonResult(FF_MAYBE_UNUSED FFCameraOptions* options, yyjson
     }
 }
 
-void ffPrintCameraHelpFormat(void)
-{
-    FF_PRINT_MODULE_FORMAT_HELP_CHECKED(FF_CAMERA_MODULE_NAME, "{1} ({4}px x {5}px)", FF_CAMERA_NUM_FORMAT_ARGS, ((const char* []) {
-        "Device name - name",
-        "Vendor - vendor",
-        "Color space - colorspace",
-        "Identifier - id",
-        "Width (in px) - width",
-        "Height (in px) - height",
-    }));
-}
-
 void ffInitCameraOptions(FFCameraOptions* options)
 {
-    ffOptionInitModuleBaseInfo(
-        &options->moduleInfo,
-        FF_CAMERA_MODULE_NAME,
-        "Print available cameras",
-        ffParseCameraCommandOptions,
-        ffParseCameraJsonObject,
-        ffPrintCamera,
-        ffGenerateCameraJsonResult,
-        ffPrintCameraHelpFormat,
-        ffGenerateCameraJsonConfig
-    );
-    ffOptionInitModuleArg(&options->moduleArgs);
+    ffOptionInitModuleArg(&options->moduleArgs, "󰄀");
 }
 
 void ffDestroyCameraOptions(FFCameraOptions* options)
 {
     ffOptionDestroyModuleArg(&options->moduleArgs);
 }
+
+FFModuleBaseInfo ffCameraModuleInfo = {
+    .name = FF_CAMERA_MODULE_NAME,
+    .description = "Print available cameras",
+    .initOptions = (void*) ffInitCameraOptions,
+    .destroyOptions = (void*) ffDestroyCameraOptions,
+    .parseJsonObject = (void*) ffParseCameraJsonObject,
+    .printModule = (void*) ffPrintCamera,
+    .generateJsonResult = (void*) ffGenerateCameraJsonResult,
+    .generateJsonConfig = (void*) ffGenerateCameraJsonConfig,
+    .formatArgs = FF_FORMAT_ARG_LIST(((FFModuleFormatArg[]) {
+        {"Device name", "name"},
+        {"Vendor", "vendor"},
+        {"Color space", "colorspace"},
+        {"Identifier", "id"},
+        {"Width (in px)", "width"},
+        {"Height (in px)", "height"},
+    }))
+};
