@@ -1,6 +1,6 @@
 #pragma once
 
-#include "fastfetch.h"
+#include "option.h"
 
 #define FF_CPUUSAGE_MODULE_NAME "CPUUsage"
 
@@ -9,3 +9,5 @@ void ffPrepareCPUUsage();
 void ffPrintCPUUsage(FFCPUUsageOptions* options);
 void ffInitCPUUsageOptions(FFCPUUsageOptions* options);
 void ffDestroyCPUUsageOptions(FFCPUUsageOptions* options);
+
+extern FFModuleBaseInfo ffCPUUsageModuleInfo;
