@@ -9,7 +9,6 @@
 #pragma GCC diagnostic ignored "-Wformat" // warning: unknown conversion type character 'F' in format
 
 #define FF_DATETIME_DISPLAY_NAME "Date & Time"
-#define FF_DATETIME_NUM_FORMAT_ARGS 23
 
 typedef struct FFDateTimeResult
 {
@@ -55,7 +54,7 @@ void ffPrintDateTimeFormat(struct tm* tm, const FFModuleArgs* moduleArgs)
     result.dayInYear = (uint8_t) (tm->tm_yday + 1);
     result.dayInMonth = (uint8_t) tm->tm_mday;
     result.dayInWeek = tm->tm_wday == 0 ? 7 : (uint8_t) tm->tm_wday;
-    strftime(result.dayPretty, sizeof(result.dayPretty), "%0d", tm);
+    strftime(result.dayPretty, sizeof(result.dayPretty), "%d", tm);
     result.hour = (uint8_t) tm->tm_hour;
     strftime(result.hourPretty, sizeof(result.hourPretty), "%H", tm);
     result.hour12 = (uint8_t) (result.hour % 12);
@@ -67,7 +66,7 @@ void ffPrintDateTimeFormat(struct tm* tm, const FFModuleArgs* moduleArgs)
     strftime(result.offsetFromUtc, sizeof(result.offsetFromUtc), "%z", tm);
     strftime(result.timezoneName, sizeof(result.timezoneName), "%Z", tm);
 
-    FF_PRINT_FORMAT_CHECKED(FF_DATETIME_DISPLAY_NAME, 0, moduleArgs, FF_PRINT_TYPE_DEFAULT, FF_DATETIME_NUM_FORMAT_ARGS, ((FFformatarg[]) {
+    FF_PRINT_FORMAT_CHECKED(FF_DATETIME_DISPLAY_NAME, 0, moduleArgs, FF_PRINT_TYPE_DEFAULT, ((FFformatarg[]) {
         FF_FORMAT_ARG(result.year, "year"), // 1
         FF_FORMAT_ARG(result.yearShort, "year-short"), // 2
         FF_FORMAT_ARG(result.month, "month"), // 3
@@ -118,39 +117,22 @@ void ffPrintDateTime(FFDateTimeOptions* options)
     puts(buffer);
 }
 
-bool ffParseDateTimeCommandOptions(FFDateTimeOptions* options, const char* key, const char* value)
-{
-    const char* subKey = ffOptionTestPrefix(key, FF_DATETIME_MODULE_NAME);
-    if (!subKey) return false;
-    if (ffOptionParseModuleArgs(key, subKey, value, &options->moduleArgs))
-        return true;
-
-    return false;
-}
-
 void ffParseDateTimeJsonObject(FFDateTimeOptions* options, yyjson_val* module)
 {
-    yyjson_val *key_, *val;
+    yyjson_val *key, *val;
     size_t idx, max;
-    yyjson_obj_foreach(module, idx, max, key_, val)
+    yyjson_obj_foreach(module, idx, max, key, val)
     {
-        const char* key = yyjson_get_str(key_);
-        if(ffStrEqualsIgnCase(key, "type"))
-            continue;
-
         if (ffJsonConfigParseModuleArgs(key, val, &options->moduleArgs))
             continue;
 
-        ffPrintError(FF_DATETIME_DISPLAY_NAME, 0, &options->moduleArgs, FF_PRINT_TYPE_DEFAULT, "Unknown JSON key %s", key);
+        ffPrintError(FF_DATETIME_DISPLAY_NAME, 0, &options->moduleArgs, FF_PRINT_TYPE_DEFAULT, "Unknown JSON key %s", unsafe_yyjson_get_str(key));
     }
 }
 
 void ffGenerateDateTimeJsonConfig(FFDateTimeOptions* options, yyjson_mut_doc* doc, yyjson_mut_val* module)
 {
-    __attribute__((__cleanup__(ffDestroyDateTimeOptions))) FFDateTimeOptions defaultOptions;
-    ffInitDateTimeOptions(&defaultOptions);
-
-    ffJsonConfigGenerateModuleArgsConfig(doc, module, &defaultOptions.moduleArgs, &options->moduleArgs);
+    ffJsonConfigGenerateModuleArgsConfig(doc, module, &options->moduleArgs);
 }
 
 void ffGenerateDateTimeJsonResult(FF_MAYBE_UNUSED FFDateTimeOptions* options, yyjson_mut_doc* doc, yyjson_mut_val* module)
@@ -158,48 +140,8 @@ void ffGenerateDateTimeJsonResult(FF_MAYBE_UNUSED FFDateTimeOptions* options, yy
     yyjson_mut_obj_add_strcpy(doc, module, "result", ffTimeToFullStr(ffTimeGetNow()));
 }
 
-void ffPrintDateTimeHelpFormat(void)
-{
-    FF_PRINT_MODULE_FORMAT_HELP_CHECKED(FF_DATETIME_MODULE_NAME, "{1}-{4}-{23} {14}:{18}:{20}", FF_DATETIME_NUM_FORMAT_ARGS, ((const char* []) {
-        "year - year",
-        "last two digits of year - year-short",
-        "month - month",
-        "month with leading zero - month-pretty",
-        "month name - month-name",
-        "month name short - month-name-short",
-        "week number on year - week",
-        "weekday - weekday",
-        "weekday short - weekday-short",
-        "day in year - day-in-year",
-        "day in month - day-in-month",
-        "day in week - day-in-week",
-        "hour - hour",
-        "hour with leading zero - hour-pretty",
-        "hour 12h format - hour-12",
-        "hour 12h format with leading zero - hour-12-pretty",
-        "minute - minute",
-        "minute with leading zero - minute-pretty",
-        "second - second",
-        "second with leading zero - second-pretty",
-        "offset from UTC in the ISO 8601 format - offset-from-utc",
-        "locale-dependent timezone name or abbreviation - timezone-name",
-        "day in month with leading zero - day-pretty",
-    }));
-}
-
 void ffInitDateTimeOptions(FFDateTimeOptions* options)
 {
-    ffOptionInitModuleBaseInfo(
-        &options->moduleInfo,
-        FF_DATETIME_MODULE_NAME,
-        "Print current date and time",
-        ffParseDateTimeCommandOptions,
-        ffParseDateTimeJsonObject,
-        ffPrintDateTime,
-        ffGenerateDateTimeJsonResult,
-        ffPrintDateTimeHelpFormat,
-        ffGenerateDateTimeJsonConfig
-    );
     ffOptionInitModuleArg(&options->moduleArgs, "");
 }
 
@@ -207,3 +149,39 @@ void ffDestroyDateTimeOptions(FFDateTimeOptions* options)
 {
     ffOptionDestroyModuleArg(&options->moduleArgs);
 }
+
+FFModuleBaseInfo ffDateTimeModuleInfo = {
+    .name = FF_DATETIME_MODULE_NAME,
+    .description = "Print current date and time",
+    .initOptions = (void*) ffInitDateTimeOptions,
+    .destroyOptions = (void*) ffDestroyDateTimeOptions,
+    .parseJsonObject = (void*) ffParseDateTimeJsonObject,
+    .printModule = (void*) ffPrintDateTime,
+    .generateJsonResult = (void*) ffGenerateDateTimeJsonResult,
+    .generateJsonConfig = (void*) ffGenerateDateTimeJsonConfig,
+    .formatArgs = FF_FORMAT_ARG_LIST(((FFModuleFormatArg[]) {
+        {"Year", "year"},
+        {"Last two digits of year", "year-short"},
+        {"Month", "month"},
+        {"Month with leading zero", "month-pretty"},
+        {"Month name", "month-name"},
+        {"Month name short", "month-name-short"},
+        {"Week number on year", "week"},
+        {"Weekday", "weekday"},
+        {"Weekday short", "weekday-short"},
+        {"Day in year", "day-in-year"},
+        {"Day in month", "day-in-month"},
+        {"Day in week", "day-in-week"},
+        {"Hour", "hour"},
+        {"Hour with leading zero", "hour-pretty"},
+        {"Hour 12h format", "hour-12"},
+        {"Hour 12h format with leading zero", "hour-12-pretty"},
+        {"Minute", "minute"},
+        {"Minute with leading zero", "minute-pretty"},
+        {"Second", "second"},
+        {"Second with leading zero", "second-pretty"},
+        {"Offset from UTC in the ISO 8601 format", "offset-from-utc"},
+        {"Locale-dependent timezone name or abbreviation", "timezone-name"},
+        {"Day in month with leading zero", "day-pretty"},
+    }))
+};
