@@ -1,13 +1,12 @@
 #include "common/printing.h"
 #include "common/jsonconfig.h"
+#include "common/size.h"
 #include "detection/displayserver/displayserver.h"
 #include "modules/display/display.h"
 #include "util/stringUtils.h"
 
 #include <math.h>
 
-#define FF_DISPLAY_NUM_FORMAT_ARGS 19
-
 static int sortByNameAsc(FFDisplayResult* a, FFDisplayResult* b)
 {
     return ffStrbufComp(&a->name, &b->name);
@@ -53,10 +52,11 @@ void ffPrintDisplay(FFDisplayOptions* options)
             {
                 if (result->refreshRate > 0)
                 {
+                    const char* space = instance.config.display.freqSpaceBeforeUnit == FF_SPACE_BEFORE_UNIT_ALWAYS ? " " : "";
                     if (options->preciseRefreshRate)
-                        ffStrbufAppendF(&buffer, " @ %gHz", result->refreshRate);
+                        ffStrbufAppendF(&buffer, " @ %g%sHz", result->refreshRate, space);
                     else
-                        ffStrbufAppendF(&buffer, " @ %iHz", (uint32_t) (result->refreshRate + 0.5));
+                        ffStrbufAppendF(&buffer, " @ %i%sHz", (uint32_t) (result->refreshRate + 0.5), space);
                 }
                 ffStrbufAppendS(&buffer, ", ");
             }
@@ -77,7 +77,7 @@ void ffPrintDisplay(FFDisplayOptions* options)
     {
         FFDisplayResult* result = FF_LIST_GET(FFDisplayResult, dsResult->displays, i);
         uint32_t moduleIndex = dsResult->displays.length == 1 ? 0 : i + 1;
-        const char* displayType = result->type == FF_DISPLAY_TYPE_UNKNOWN ? NULL : result->type == FF_DISPLAY_TYPE_BUILTIN ? "built-in" : "external";
+        const char* displayType = result->type == FF_DISPLAY_TYPE_UNKNOWN ? NULL : result->type == FF_DISPLAY_TYPE_BUILTIN ? "Built-in" : "External";
 
         ffStrbufClear(&key);
         if(options->moduleArgs.key.length == 0)
@@ -91,7 +91,7 @@ void ffPrintDisplay(FFDisplayOptions* options)
         }
         else
         {
-            FF_PARSE_FORMAT_STRING_CHECKED(&key, &options->moduleArgs.key, 4, ((FFformatarg[]){
+            FF_PARSE_FORMAT_STRING_CHECKED(&key, &options->moduleArgs.key, ((FFformatarg[]) {
                 FF_FORMAT_ARG(moduleIndex, "index"),
                 FF_FORMAT_ARG(result->name, "name"),
                 FF_FORMAT_ARG(displayType, "type"),
@@ -110,10 +110,11 @@ void ffPrintDisplay(FFDisplayOptions* options)
 
             if(result->refreshRate > 0)
             {
+                const char* space = instance.config.display.freqSpaceBeforeUnit == FF_SPACE_BEFORE_UNIT_NEVER ? "" : " ";
                 if(options->preciseRefreshRate)
-                    ffStrbufAppendF(&buffer, " @ %g Hz", ((int) (result->refreshRate * 1000 + 0.5)) / 1000.0);
+                    ffStrbufAppendF(&buffer, " @ %g%sHz", ((int) (result->refreshRate * 1000 + 0.5)) / 1000.0, space);
                 else
-                    ffStrbufAppendF(&buffer, " @ %i Hz", (uint32_t) (result->refreshRate + 0.5));
+                    ffStrbufAppendF(&buffer, " @ %i%sHz", (uint32_t) (result->refreshRate + 0.5), space);
             }
 
             if(
@@ -124,11 +125,21 @@ void ffPrintDisplay(FFDisplayOptions* options)
             if (inch > 1)
                 ffStrbufAppendF(&buffer, " in %i\"", (uint32_t) (inch + 0.5));
 
-            if(result->type != FF_DISPLAY_TYPE_UNKNOWN)
-                ffStrbufAppendS(&buffer, result->type == FF_DISPLAY_TYPE_BUILTIN ? " [Built-in]" : " [External]");
+            bool flag = false;
+            if (result->type != FF_DISPLAY_TYPE_UNKNOWN)
+            {
+                ffStrbufAppendS(&buffer, result->type == FF_DISPLAY_TYPE_BUILTIN ? " [Built-in" : " [External");
+                flag = true;
+            }
 
             if (result->hdrStatus == FF_DISPLAY_HDR_STATUS_ENABLED)
-                ffStrbufAppendS(&buffer, " [HDR]");
+            {
+                ffStrbufAppendS(&buffer, flag ? ", HDR" : " [HDR");
+                flag = true;
+            }
+
+            if (flag)
+                ffStrbufAppendS(&buffer, "]");
 
             if(moduleIndex > 0 && result->primary)
                 ffStrbufAppendS(&buffer, " *");
@@ -138,8 +149,9 @@ void ffPrintDisplay(FFDisplayOptions* options)
         }
         else
         {
-            double ppi = sqrt(result->width * result->width + result->height * result->height) / inch;
+            double ppi = inch == 0 ? 0 : sqrt(result->width * result->width + result->height * result->height) / inch;
             bool hdrEnabled = result->hdrStatus == FF_DISPLAY_HDR_STATUS_ENABLED;
+            bool hdrCompatible = result->hdrStatus == FF_DISPLAY_HDR_STATUS_SUPPORTED || result->hdrStatus == FF_DISPLAY_HDR_STATUS_ENABLED;
             uint32_t iInch = (uint32_t) (inch + 0.5), iPpi = (uint32_t) (ppi + 0.5);
 
             char refreshRate[16];
@@ -153,6 +165,17 @@ void ffPrintDisplay(FFDisplayOptions* options)
             else
                 refreshRate[0] = 0;
 
+            char preferredRefreshRate[16];
+            if(result->preferredRefreshRate > 0)
+            {
+                if(options->preciseRefreshRate)
+                    snprintf(preferredRefreshRate, ARRAY_SIZE(preferredRefreshRate), "%g", ((int) (result->preferredRefreshRate * 1000 + 0.5)) / 1000.0);
+                else
+                    snprintf(preferredRefreshRate, ARRAY_SIZE(preferredRefreshRate), "%i", (uint32_t) (result->preferredRefreshRate + 0.5));
+            }
+            else
+                preferredRefreshRate[0] = 0;
+
             char buf[32];
             if (result->serial)
             {
@@ -162,7 +185,9 @@ void ffPrintDisplay(FFDisplayOptions* options)
             else
                 buf[0] = '\0';
 
-            FF_PRINT_FORMAT_CHECKED(key.chars, 0, &options->moduleArgs, FF_PRINT_TYPE_NO_CUSTOM_KEY, FF_DISPLAY_NUM_FORMAT_ARGS, ((FFformatarg[]) {
+            double scaleFactor = (double) result->height / (double) result->scaledHeight;
+
+            FF_PRINT_FORMAT_CHECKED(key.chars, 0, &options->moduleArgs, FF_PRINT_TYPE_NO_CUSTOM_KEY, ((FFformatarg[]) {
                 FF_FORMAT_ARG(result->width, "width"),
                 FF_FORMAT_ARG(result->height, "height"),
                 FF_FORMAT_ARG(refreshRate, "refresh-rate"),
@@ -182,139 +207,116 @@ void ffPrintDisplay(FFDisplayOptions* options)
                 FF_FORMAT_ARG(result->manufactureWeek, "manufacture-week"),
                 FF_FORMAT_ARG(buf, "serial"),
                 FF_FORMAT_ARG(result->platformApi, "platform-api"),
+                FF_FORMAT_ARG(hdrCompatible, "hdr-compatible"),
+                FF_FORMAT_ARG(scaleFactor, "scale-factor"),
+                FF_FORMAT_ARG(result->preferredWidth, "preferred-width"),
+                FF_FORMAT_ARG(result->preferredHeight, "preferred-height"),
+                FF_FORMAT_ARG(preferredRefreshRate, "preferred-refresh-rate"),
             }));
         }
     }
 }
 
-bool ffParseDisplayCommandOptions(FFDisplayOptions* options, const char* key, const char* value)
-{
-    const char* subKey = ffOptionTestPrefix(key, FF_DISPLAY_MODULE_NAME);
-    if (!subKey) return false;
-    if (ffOptionParseModuleArgs(key, subKey, value, &options->moduleArgs))
-        return true;
-
-    if (ffStrEqualsIgnCase(subKey, "compact-type"))
-    {
-        options->compactType = (FFDisplayCompactType) ffOptionParseEnum(key, value, (FFKeyValuePair[]) {
-            { "none", FF_DISPLAY_COMPACT_TYPE_NONE },
-            { "original", FF_DISPLAY_COMPACT_TYPE_ORIGINAL_BIT },
-            { "scaled", FF_DISPLAY_COMPACT_TYPE_SCALED_BIT },
-            { "original-with-refresh-rate", FF_DISPLAY_COMPACT_TYPE_ORIGINAL_BIT | FF_DISPLAY_COMPACT_TYPE_REFRESH_RATE_BIT },
-            { "scaled-with-refresh-rate", FF_DISPLAY_COMPACT_TYPE_SCALED_BIT | FF_DISPLAY_COMPACT_TYPE_REFRESH_RATE_BIT },
-            {},
-        });
-        return true;
-    }
-
-    if (ffStrEqualsIgnCase(subKey, "precise-refresh-rate"))
-    {
-        options->preciseRefreshRate = ffOptionParseBoolean(value);
-        return true;
-    }
-
-    if (ffStrEqualsIgnCase(subKey, "order"))
-    {
-        options->order = (FFDisplayOrder) ffOptionParseEnum(key, value, (FFKeyValuePair[]) {
-            { "asc", FF_DISPLAY_ORDER_ASC },
-            { "desc", FF_DISPLAY_ORDER_DESC },
-            { "none", FF_DISPLAY_ORDER_NONE },
-            {},
-        });
-        return true;
-    }
-
-    return false;
-}
-
 void ffParseDisplayJsonObject(FFDisplayOptions* options, yyjson_val* module)
 {
-    yyjson_val *key_, *val;
+    yyjson_val *key, *val;
     size_t idx, max;
-    yyjson_obj_foreach(module, idx, max, key_, val)
+    yyjson_obj_foreach(module, idx, max, key, val)
     {
-        const char* key = yyjson_get_str(key_);
-        if(ffStrEqualsIgnCase(key, "type"))
-            continue;
-
         if (ffJsonConfigParseModuleArgs(key, val, &options->moduleArgs))
             continue;
 
-        if (ffStrEqualsIgnCase(key, "compactType"))
+        if (unsafe_yyjson_equals_str(key, "compactType"))
         {
-            int value;
-            const char* error = ffJsonConfigParseEnum(val, &value, (FFKeyValuePair[]) {
-                { "none", FF_DISPLAY_COMPACT_TYPE_NONE },
-                { "original", FF_DISPLAY_COMPACT_TYPE_ORIGINAL_BIT },
-                { "scaled", FF_DISPLAY_COMPACT_TYPE_SCALED_BIT },
-                { "original-with-refresh-rate", FF_DISPLAY_COMPACT_TYPE_ORIGINAL_BIT | FF_DISPLAY_COMPACT_TYPE_REFRESH_RATE_BIT },
-                { "scaled-with-refresh-rate", FF_DISPLAY_COMPACT_TYPE_SCALED_BIT | FF_DISPLAY_COMPACT_TYPE_REFRESH_RATE_BIT },
-                {},
-            });
-            if (error)
-                ffPrintError(FF_DISPLAY_MODULE_NAME, 0, &options->moduleArgs, FF_PRINT_TYPE_DEFAULT, "Invalid %s value: %s", key, error);
+            if (yyjson_is_null(val))
+                options->compactType = FF_DISPLAY_COMPACT_TYPE_NONE;
             else
-                options->compactType = (FFDisplayCompactType) value;
+            {
+                int value;
+                const char* error = ffJsonConfigParseEnum(val, &value, (FFKeyValuePair[]) {
+                    { "none", FF_DISPLAY_COMPACT_TYPE_NONE },
+                    { "original", FF_DISPLAY_COMPACT_TYPE_ORIGINAL_BIT },
+                    { "scaled", FF_DISPLAY_COMPACT_TYPE_SCALED_BIT },
+                    { "original-with-refresh-rate", FF_DISPLAY_COMPACT_TYPE_ORIGINAL_BIT | FF_DISPLAY_COMPACT_TYPE_REFRESH_RATE_BIT },
+                    { "scaled-with-refresh-rate", FF_DISPLAY_COMPACT_TYPE_SCALED_BIT | FF_DISPLAY_COMPACT_TYPE_REFRESH_RATE_BIT },
+                    {},
+                });
+                if (error)
+                    ffPrintError(FF_DISPLAY_MODULE_NAME, 0, &options->moduleArgs, FF_PRINT_TYPE_DEFAULT, "Invalid %s value: %s", unsafe_yyjson_get_str(key), error);
+                else
+                    options->compactType = (FFDisplayCompactType) value;
+            }
             continue;
         }
 
-        if (ffStrEqualsIgnCase(key, "preciseRefreshRate"))
+        if (unsafe_yyjson_equals_str(key, "preciseRefreshRate"))
         {
             options->preciseRefreshRate = yyjson_get_bool(val);
             continue;
         }
 
-        if (ffStrEqualsIgnCase(key, "order"))
+        if (unsafe_yyjson_equals_str(key, "order"))
         {
-            int value;
-            const char* error = ffJsonConfigParseEnum(val, &value, (FFKeyValuePair[]) {
-                { "asc", FF_DISPLAY_ORDER_ASC },
-                { "desc", FF_DISPLAY_ORDER_DESC },
-                { "none", FF_DISPLAY_ORDER_NONE },
-                {},
-            });
-            if (error)
-                ffPrintError(FF_DISPLAY_MODULE_NAME, 0, &options->moduleArgs, FF_PRINT_TYPE_DEFAULT, "Invalid %s value: %s", key, error);
+            if (yyjson_is_null(val))
+                options->order = FF_DISPLAY_ORDER_NONE;
             else
-                options->order = (FFDisplayOrder) value;
+            {
+                int value;
+                const char* error = ffJsonConfigParseEnum(val, &value, (FFKeyValuePair[]) {
+                    { "asc", FF_DISPLAY_ORDER_ASC },
+                    { "desc", FF_DISPLAY_ORDER_DESC },
+                    { "none", FF_DISPLAY_ORDER_NONE },
+                    {},
+                });
+                if (error)
+                    ffPrintError(FF_DISPLAY_MODULE_NAME, 0, &options->moduleArgs, FF_PRINT_TYPE_DEFAULT, "Invalid %s value: %s", unsafe_yyjson_get_str(key), error);
+                else
+                    options->order = (FFDisplayOrder) value;
+            }
             continue;
         }
 
-        ffPrintError(FF_DISPLAY_MODULE_NAME, 0, &options->moduleArgs, FF_PRINT_TYPE_DEFAULT, "Unknown JSON key %s", key);
+        ffPrintError(FF_DISPLAY_MODULE_NAME, 0, &options->moduleArgs, FF_PRINT_TYPE_DEFAULT, "Unknown JSON key %s", unsafe_yyjson_get_str(key));
     }
 }
 
 void ffGenerateDisplayJsonConfig(FFDisplayOptions* options, yyjson_mut_doc* doc, yyjson_mut_val* module)
 {
-    __attribute__((__cleanup__(ffDestroyDisplayOptions))) FFDisplayOptions defaultOptions;
-    ffInitDisplayOptions(&defaultOptions);
+    ffJsonConfigGenerateModuleArgsConfig(doc, module, &options->moduleArgs);
 
-    ffJsonConfigGenerateModuleArgsConfig(doc, module, &defaultOptions.moduleArgs, &options->moduleArgs);
-
-    if (options->compactType != defaultOptions.compactType)
+    switch ((int) options->compactType)
     {
-        switch ((int) options->compactType)
-        {
-            case FF_DISPLAY_COMPACT_TYPE_NONE:
-                yyjson_mut_obj_add_str(doc, module, "compactType", "none");
-                break;
-            case FF_DISPLAY_COMPACT_TYPE_ORIGINAL_BIT:
-                yyjson_mut_obj_add_str(doc, module, "compactType", "original");
-                break;
-            case FF_DISPLAY_COMPACT_TYPE_SCALED_BIT:
-                yyjson_mut_obj_add_str(doc, module, "compactType", "scaled");
-                break;
-            case FF_DISPLAY_COMPACT_TYPE_ORIGINAL_BIT | FF_DISPLAY_COMPACT_TYPE_REFRESH_RATE_BIT:
-                yyjson_mut_obj_add_str(doc, module, "compactType", "original-with-refresh-rate");
-                break;
-            case FF_DISPLAY_COMPACT_TYPE_SCALED_BIT | FF_DISPLAY_COMPACT_TYPE_REFRESH_RATE_BIT:
-                yyjson_mut_obj_add_str(doc, module, "compactType", "scaled-with-refresh-rate");
-                break;
-        }
+        case FF_DISPLAY_COMPACT_TYPE_NONE:
+            yyjson_mut_obj_add_str(doc, module, "compactType", "none");
+            break;
+        case FF_DISPLAY_COMPACT_TYPE_ORIGINAL_BIT:
+            yyjson_mut_obj_add_str(doc, module, "compactType", "original");
+            break;
+        case FF_DISPLAY_COMPACT_TYPE_SCALED_BIT:
+            yyjson_mut_obj_add_str(doc, module, "compactType", "scaled");
+            break;
+        case FF_DISPLAY_COMPACT_TYPE_ORIGINAL_BIT | FF_DISPLAY_COMPACT_TYPE_REFRESH_RATE_BIT:
+            yyjson_mut_obj_add_str(doc, module, "compactType", "original-with-refresh-rate");
+            break;
+        case FF_DISPLAY_COMPACT_TYPE_SCALED_BIT | FF_DISPLAY_COMPACT_TYPE_REFRESH_RATE_BIT:
+            yyjson_mut_obj_add_str(doc, module, "compactType", "scaled-with-refresh-rate");
+            break;
     }
 
-    if (options->preciseRefreshRate != defaultOptions.preciseRefreshRate)
-        yyjson_mut_obj_add_bool(doc, module, "preciseRefreshRate", options->preciseRefreshRate);
+    yyjson_mut_obj_add_bool(doc, module, "preciseRefreshRate", options->preciseRefreshRate);
+
+    switch (options->order)
+    {
+        case FF_DISPLAY_ORDER_NONE:
+            yyjson_mut_obj_add_null(doc, module, "order");
+            break;
+        case FF_DISPLAY_ORDER_ASC:
+            yyjson_mut_obj_add_str(doc, module, "order", "asc");
+            break;
+        case FF_DISPLAY_ORDER_DESC:
+            yyjson_mut_obj_add_str(doc, module, "order", "desc");
+            break;
+    }
 }
 
 void ffGenerateDisplayJsonResult(FF_MAYBE_UNUSED FFDisplayOptions* options, yyjson_mut_doc* doc, yyjson_mut_val* module)
@@ -338,16 +340,21 @@ void ffGenerateDisplayJsonResult(FF_MAYBE_UNUSED FFDisplayOptions* options, yyjs
         yyjson_mut_val* output = yyjson_mut_obj_add_obj(doc, obj, "output");
         yyjson_mut_obj_add_uint(doc, output, "width", item->width);
         yyjson_mut_obj_add_uint(doc, output, "height", item->height);
+        yyjson_mut_obj_add_real(doc, output, "refreshRate", item->refreshRate);
 
         yyjson_mut_val* scaled = yyjson_mut_obj_add_obj(doc, obj, "scaled");
         yyjson_mut_obj_add_uint(doc, scaled, "width", item->scaledWidth);
         yyjson_mut_obj_add_uint(doc, scaled, "height", item->scaledHeight);
 
+        yyjson_mut_val* preferred = yyjson_mut_obj_add_obj(doc, obj, "preferred");
+        yyjson_mut_obj_add_uint(doc, preferred, "width", item->preferredWidth);
+        yyjson_mut_obj_add_uint(doc, preferred, "height", item->preferredHeight);
+        yyjson_mut_obj_add_real(doc, preferred, "refreshRate", item->preferredRefreshRate);
+
         yyjson_mut_val* physical = yyjson_mut_obj_add_obj(doc, obj, "physical");
         yyjson_mut_obj_add_uint(doc, physical, "width", item->physicalWidth);
         yyjson_mut_obj_add_uint(doc, physical, "height", item->physicalHeight);
 
-        yyjson_mut_obj_add_real(doc, obj, "refreshRate", item->refreshRate);
         yyjson_mut_obj_add_uint(doc, obj, "rotation", item->rotation);
         yyjson_mut_obj_add_uint(doc, obj, "bitDepth", item->bitDepth);
         if (item->hdrStatus == FF_DISPLAY_HDR_STATUS_UNKNOWN)
@@ -401,44 +408,8 @@ void ffGenerateDisplayJsonResult(FF_MAYBE_UNUSED FFDisplayOptions* options, yyjs
     }
 }
 
-void ffPrintDisplayHelpFormat(void)
-{
-    FF_PRINT_MODULE_FORMAT_HELP_CHECKED(FF_DISPLAY_MODULE_NAME, "{1}x{2} @ {3}Hz (as {4}x{5}) [{7}]", FF_DISPLAY_NUM_FORMAT_ARGS, ((const char* []) {
-        "Screen width (in pixels) - width",
-        "Screen height (in pixels) - height",
-        "Screen refresh rate (in Hz) - refresh-rate",
-        "Screen scaled width (in pixels) - scaled-width",
-        "Screen scaled height (in pixels) - scaled-height",
-        "Screen name - name",
-        "Screen type (builtin, external or unknown) - type",
-        "Screen rotation (in degrees) - rotation",
-        "True if being the primary screen - is-primary",
-        "Screen physical width (in millimeters) - physical-width",
-        "Screen physical height (in millimeters) - physical-height",
-        "Physical diagonal length in inches - inch",
-        "Pixels per inch (PPI) - ppi",
-        "Bits per color channel - bit-depth",
-        "True if high dynamic range (HDR) mode is enabled - hdr-enabled",
-        "Year of manufacturing - manufacture-year",
-        "Nth week of manufacturing in the year - manufacture-week",
-        "Serial number - serial",
-        "The platform API used when detecting the display - platform-api",
-    }));
-}
-
 void ffInitDisplayOptions(FFDisplayOptions* options)
 {
-    ffOptionInitModuleBaseInfo(
-        &options->moduleInfo,
-        FF_DISPLAY_MODULE_NAME,
-        "Print resolutions, refresh rates, etc",
-        ffParseDisplayCommandOptions,
-        ffParseDisplayJsonObject,
-        ffPrintDisplay,
-        ffGenerateDisplayJsonResult,
-        ffPrintDisplayHelpFormat,
-        ffGenerateDisplayJsonConfig
-    );
     ffOptionInitModuleArg(&options->moduleArgs, "󰍹");
     options->compactType = FF_DISPLAY_COMPACT_TYPE_NONE;
     options->preciseRefreshRate = false;
@@ -448,3 +419,40 @@ void ffDestroyDisplayOptions(FFDisplayOptions* options)
 {
     ffOptionDestroyModuleArg(&options->moduleArgs);
 }
+
+FFModuleBaseInfo ffDisplayModuleInfo = {
+    .name = FF_DISPLAY_MODULE_NAME,
+    .description = "Print resolutions, refresh rates, etc",
+    .initOptions = (void*) ffInitDisplayOptions,
+    .destroyOptions = (void*) ffDestroyDisplayOptions,
+    .parseJsonObject = (void*) ffParseDisplayJsonObject,
+    .printModule = (void*) ffPrintDisplay,
+    .generateJsonResult = (void*) ffGenerateDisplayJsonResult,
+    .generateJsonConfig = (void*) ffGenerateDisplayJsonConfig,
+    .formatArgs = FF_FORMAT_ARG_LIST(((FFModuleFormatArg[]) {
+        {"Screen configured width (in pixels)", "width"},
+        {"Screen configured height (in pixels)", "height"},
+        {"Screen configured refresh rate (in Hz)", "refresh-rate"},
+        {"Screen scaled width (in pixels)", "scaled-width"},
+        {"Screen scaled height (in pixels)", "scaled-height"},
+        {"Screen name", "name"},
+        {"Screen type (Built-in or External)", "type"},
+        {"Screen rotation (in degrees)", "rotation"},
+        {"True if being the primary screen", "is-primary"},
+        {"Screen physical width (in millimeters)", "physical-width"},
+        {"Screen physical height (in millimeters)", "physical-height"},
+        {"Physical diagonal length in inches", "inch"},
+        {"Pixels per inch (PPI)", "ppi"},
+        {"Bits per color channel", "bit-depth"},
+        {"True if high dynamic range (HDR) mode is enabled", "hdr-enabled"},
+        {"Year of manufacturing", "manufacture-year"},
+        {"Nth week of manufacturing in the year", "manufacture-week"},
+        {"Serial number", "serial"},
+        {"The platform API used when detecting the display", "platform-api"},
+        {"True if the display is HDR compatible", "hdr-compatible"},
+        {"HiDPI scale factor", "scale-factor"},
+        {"Screen preferred width (in pixels)", "preferred-width"},
+        {"Screen preferred height (in pixels)", "preferred-height"},
+        {"Screen preferred refresh rate (in Hz)", "preferred-refresh-rate"},
+    }))
+};
