@@ -86,39 +86,22 @@ void ffPrintMonitor(FFMonitorOptions* options)
     }
 }
 
-bool ffParseMonitorCommandOptions(FFMonitorOptions* options, const char* key, const char* value)
-{
-    const char* subKey = ffOptionTestPrefix(key, FF_MONITOR_MODULE_NAME);
-    if (!subKey) return false;
-    if (ffOptionParseModuleArgs(key, subKey, value, &options->moduleArgs))
-        return true;
-
-    return false;
-}
-
 void ffParseMonitorJsonObject(FFMonitorOptions* options, yyjson_val* module)
 {
-    yyjson_val *key_, *val;
+    yyjson_val *key, *val;
     size_t idx, max;
-    yyjson_obj_foreach(module, idx, max, key_, val)
+    yyjson_obj_foreach(module, idx, max, key, val)
     {
-        const char* key = yyjson_get_str(key_);
-        if(ffStrEqualsIgnCase(key, "type"))
-            continue;
-
         if (ffJsonConfigParseModuleArgs(key, val, &options->moduleArgs))
             continue;
 
-        ffPrintError(FF_MONITOR_MODULE_NAME, 0, &options->moduleArgs, FF_PRINT_TYPE_DEFAULT, "Unknown JSON key %s", key);
+        ffPrintError(FF_MONITOR_MODULE_NAME, 0, &options->moduleArgs, FF_PRINT_TYPE_DEFAULT, "Unknown JSON key %s", unsafe_yyjson_get_str(key));
     }
 }
 
 void ffGenerateMonitorJsonConfig(FFMonitorOptions* options, yyjson_mut_doc* doc, yyjson_mut_val* module)
 {
-    __attribute__((__cleanup__(ffDestroyMonitorOptions))) FFMonitorOptions defaultOptions;
-    ffInitMonitorOptions(&defaultOptions);
-
-    ffJsonConfigGenerateModuleArgsConfig(doc, module, &defaultOptions.moduleArgs, &options->moduleArgs);
+    ffJsonConfigGenerateModuleArgsConfig(doc, module, &options->moduleArgs);
 }
 
 void ffGenerateMonitorJsonResult(FF_MAYBE_UNUSED FFMonitorOptions* options, yyjson_mut_doc* doc, yyjson_mut_val* module)
@@ -126,10 +109,21 @@ void ffGenerateMonitorJsonResult(FF_MAYBE_UNUSED FFMonitorOptions* options, yyjs
     yyjson_mut_obj_add_str(doc, module, "error", "Monitor module is an alias of Display module");
 }
 
-static FFModuleBaseInfo ffModuleInfo = {
+void ffInitMonitorOptions(FFMonitorOptions* options)
+{
+    ffOptionInitModuleArg(&options->moduleArgs, "󰹑");
+}
+
+void ffDestroyMonitorOptions(FFMonitorOptions* options)
+{
+    ffOptionDestroyModuleArg(&options->moduleArgs);
+}
+
+FFModuleBaseInfo ffMonitorModuleInfo = {
     .name = FF_MONITOR_MODULE_NAME,
     .description = "Alias of Display module",
-    .parseCommandOptions = (void*) ffParseMonitorCommandOptions,
+    .initOptions = (void*) ffInitMonitorOptions,
+    .destroyOptions = (void*) ffDestroyMonitorOptions,
     .parseJsonObject = (void*) ffParseMonitorJsonObject,
     .printModule = (void*) ffPrintMonitor,
     .generateJsonResult = (void*) ffGenerateMonitorJsonResult,
@@ -149,14 +143,3 @@ static FFModuleBaseInfo ffModuleInfo = {
         {"True if the display is HDR compatible", "hdr-compatible"},
     }))
 };
-
-void ffInitMonitorOptions(FFMonitorOptions* options)
-{
-    options->moduleInfo = ffModuleInfo;
-    ffOptionInitModuleArg(&options->moduleArgs, "󰹑");
-}
-
-void ffDestroyMonitorOptions(FFMonitorOptions* options)
-{
-    ffOptionDestroyModuleArg(&options->moduleArgs);
-}
