@@ -2,7 +2,6 @@
 #include "common/jsonconfig.h"
 #include "modules/title/title.h"
 #include "util/textModifier.h"
-#include "util/stringUtils.h"
 
 static void appendText(FFstrbuf* output, const FFstrbuf* text, const FFstrbuf* color)
 {
@@ -30,6 +29,7 @@ void ffPrintTitle(FFTitleOptions* options)
     FF_STRBUF_AUTO_DESTROY hostName = ffStrbufCreateCopy(&instance.state.platform.hostName);
     if (!options->fqdn)
         ffStrbufSubstrBeforeFirstC(&hostName, '.');
+    instance.state.titleFqdn = options->fqdn;
 
     FF_STRBUF_AUTO_DESTROY hostNameColored = ffStrbufCreate();
     appendText(&hostNameColored, &hostName, &options->colorHost);
@@ -68,60 +68,22 @@ void ffPrintTitle(FFTitleOptions* options)
     }
 }
 
-bool ffParseTitleCommandOptions(FFTitleOptions* options, const char* key, const char* value)
-{
-    const char* subKey = ffOptionTestPrefix(key, FF_TITLE_MODULE_NAME);
-    if (!subKey) return false;
-    if (ffOptionParseModuleArgs(key, subKey, value, &options->moduleArgs))
-        return true;
-
-    if (ffStrEqualsIgnCase(subKey, "fqdn"))
-    {
-        options->fqdn = ffOptionParseBoolean(value);
-        return true;
-    }
-
-    if (ffStrEqualsIgnCase(subKey, "color-user"))
-    {
-        ffOptionParseColor(value, &options->colorUser);
-        return true;
-    }
-
-    if (ffStrEqualsIgnCase(subKey, "color-at"))
-    {
-        ffOptionParseColor(value, &options->colorAt);
-        return true;
-    }
-
-    if (ffStrEqualsIgnCase(subKey, "color-host"))
-    {
-        ffOptionParseColor(value, &options->colorHost);
-        return true;
-    }
-
-    return false;
-}
-
 void ffParseTitleJsonObject(FFTitleOptions* options, yyjson_val* module)
 {
-    yyjson_val *key_, *val;
+    yyjson_val *key, *val;
     size_t idx, max;
-    yyjson_obj_foreach(module, idx, max, key_, val)
+    yyjson_obj_foreach(module, idx, max, key, val)
     {
-        const char* key = yyjson_get_str(key_);
-        if(ffStrEqualsIgnCase(key, "type") || ffStrEqualsIgnCase(key, "condition"))
-            continue;
-
         if (ffJsonConfigParseModuleArgs(key, val, &options->moduleArgs))
             continue;
 
-        if (ffStrEqualsIgnCase(key, "fqdn"))
+        if (unsafe_yyjson_equals_str(key, "fqdn"))
         {
             options->fqdn = yyjson_get_bool(val);
             continue;
         }
 
-        if (ffStrEqualsIgnCase(key, "color"))
+        if (unsafe_yyjson_equals_str(key, "color"))
         {
             if (!yyjson_is_obj(val))
                 continue;
@@ -138,33 +100,20 @@ void ffParseTitleJsonObject(FFTitleOptions* options, yyjson_val* module)
             continue;
         }
 
-        ffPrintError(FF_TITLE_MODULE_NAME, 0, NULL, FF_PRINT_TYPE_NO_CUSTOM_KEY, "Unknown JSON key %s", key);
+        ffPrintError(FF_TITLE_MODULE_NAME, 0, NULL, FF_PRINT_TYPE_NO_CUSTOM_KEY, "Unknown JSON key %s", unsafe_yyjson_get_str(key));
     }
 }
 
 void ffGenerateTitleJsonConfig(FFTitleOptions* options, yyjson_mut_doc* doc, yyjson_mut_val* module)
 {
-    __attribute__((__cleanup__(ffDestroyTitleOptions))) FFTitleOptions defaultOptions;
-    ffInitTitleOptions(&defaultOptions);
+    ffJsonConfigGenerateModuleArgsConfig(doc, module, &options->moduleArgs);
 
-    ffJsonConfigGenerateModuleArgsConfig(doc, module, &defaultOptions.moduleArgs, &options->moduleArgs);
+    yyjson_mut_obj_add_bool(doc, module, "fqdn", options->fqdn);
 
-    if (defaultOptions.fqdn != options->fqdn)
-        yyjson_mut_obj_add_bool(doc, module, "fqdn", options->fqdn);
-
-    yyjson_mut_val* color = yyjson_mut_obj(doc);
-
-    if (!ffStrbufEqual(&options->colorUser, &defaultOptions.colorUser))
-        yyjson_mut_obj_add_strbuf(doc, color, "user", &options->colorUser);
-
-    if (!ffStrbufEqual(&options->colorAt, &defaultOptions.colorAt))
-        yyjson_mut_obj_add_strbuf(doc, color, "at", &options->colorAt);
-
-    if (!ffStrbufEqual(&options->colorHost, &defaultOptions.colorHost))
-        yyjson_mut_obj_add_strbuf(doc, color, "host", &options->colorHost);
-
-    if (yyjson_mut_obj_size(color))
-        yyjson_mut_obj_add_val(doc, module, "color", color);
+    yyjson_mut_val* color = yyjson_mut_obj_add_obj(doc, module, "color");
+    yyjson_mut_obj_add_strbuf(doc, color, "user", &options->colorUser);
+    yyjson_mut_obj_add_strbuf(doc, color, "at", &options->colorAt);
+    yyjson_mut_obj_add_strbuf(doc, color, "host", &options->colorHost);
 }
 
 void ffGenerateTitleJsonResult(FF_MAYBE_UNUSED FFTitleOptions* options, yyjson_mut_doc* doc, yyjson_mut_val* module)
@@ -178,30 +127,8 @@ void ffGenerateTitleJsonResult(FF_MAYBE_UNUSED FFTitleOptions* options, yyjson_m
     yyjson_mut_obj_add_strbuf(doc, obj, "userShell", &instance.state.platform.userShell);
 }
 
-static FFModuleBaseInfo ffModuleInfo = {
-    .name = FF_TITLE_MODULE_NAME,
-    .description = "Print title, which contains your user name, hostname",
-    .parseCommandOptions = (void*) ffParseTitleCommandOptions,
-    .parseJsonObject = (void*) ffParseTitleJsonObject,
-    .printModule = (void*) ffPrintTitle,
-    .generateJsonResult = (void*) ffGenerateTitleJsonResult,
-    .generateJsonConfig = (void*) ffGenerateTitleJsonConfig,
-    .formatArgs = FF_FORMAT_ARG_LIST(((FFModuleFormatArg[]) {
-        {"User name", "user-name"},
-        {"Host name", "host-name"},
-        {"Home directory", "home-dir"},
-        {"Executable path of current process", "exe-path"},
-        {"User's default shell", "user-shell"},
-        {"User name (colored)", "user-name-colored"},
-        {"@ symbol (colored)", "at-symbol-colored"},
-        {"Host name (colored)", "host-name-colored"},
-        {"Full user name", "full-user-name"},
-    }))
-};
-
 void ffInitTitleOptions(FFTitleOptions* options)
 {
-    options->moduleInfo = ffModuleInfo;
     ffOptionInitModuleArg(&options->moduleArgs, "");
     ffStrbufSetStatic(&options->moduleArgs.key, " ");
 
@@ -218,3 +145,25 @@ void ffDestroyTitleOptions(FFTitleOptions* options)
     ffStrbufDestroy(&options->colorAt);
     ffStrbufDestroy(&options->colorHost);
 }
+
+FFModuleBaseInfo ffTitleModuleInfo = {
+    .name = FF_TITLE_MODULE_NAME,
+    .description = "Print title, which contains your user name, hostname",
+    .initOptions = (void*) ffInitTitleOptions,
+    .destroyOptions = (void*) ffDestroyTitleOptions,
+    .parseJsonObject = (void*) ffParseTitleJsonObject,
+    .printModule = (void*) ffPrintTitle,
+    .generateJsonResult = (void*) ffGenerateTitleJsonResult,
+    .generateJsonConfig = (void*) ffGenerateTitleJsonConfig,
+    .formatArgs = FF_FORMAT_ARG_LIST(((FFModuleFormatArg[]) {
+        {"User name", "user-name"},
+        {"Host name", "host-name"},
+        {"Home directory", "home-dir"},
+        {"Executable path of current process", "exe-path"},
+        {"User's default shell", "user-shell"},
+        {"User name (colored)", "user-name-colored"},
+        {"@ symbol (colored)", "at-symbol-colored"},
+        {"Host name (colored)", "host-name-colored"},
+        {"Full user name", "full-user-name"},
+    }))
+};
