@@ -1,9 +1,11 @@
 #pragma once
 
-#include "fastfetch.h"
+#include "option.h"
 
 #define FF_UPTIME_MODULE_NAME "Uptime"
 
 void ffPrintUptime(FFUptimeOptions* options);
 void ffInitUptimeOptions(FFUptimeOptions* options);
 void ffDestroyUptimeOptions(FFUptimeOptions* options);
+
+extern FFModuleBaseInfo ffUptimeModuleInfo;
