@@ -4,8 +4,6 @@
 #include "modules/wallpaper/wallpaper.h"
 #include "util/stringUtils.h"
 
-#define FF_WALLPAPER_NUM_FORMAT_ARGS 2
-
 void ffPrintWallpaper(FFWallpaperOptions* options)
 {
     FF_STRBUF_AUTO_DESTROY fullpath = ffStrbufCreate();
@@ -35,46 +33,29 @@ void ffPrintWallpaper(FFWallpaperOptions* options)
     }
     else
     {
-        FF_PRINT_FORMAT_CHECKED(FF_WALLPAPER_MODULE_NAME, 0, &options->moduleArgs, FF_PRINT_TYPE_DEFAULT, FF_WALLPAPER_NUM_FORMAT_ARGS, ((FFformatarg[]){
-            {FF_FORMAT_ARG_TYPE_STRING, filename},
-            {FF_FORMAT_ARG_TYPE_STRBUF, &fullpath},
+        FF_PRINT_FORMAT_CHECKED(FF_WALLPAPER_MODULE_NAME, 0, &options->moduleArgs, FF_PRINT_TYPE_DEFAULT, ((FFformatarg[]){
+            FF_FORMAT_ARG(filename, "file-name"),
+            FF_FORMAT_ARG(fullpath, "full-path"),
         }));
     }
 }
 
-bool ffParseWallpaperCommandOptions(FFWallpaperOptions* options, const char* key, const char* value)
-{
-    const char* subKey = ffOptionTestPrefix(key, FF_WALLPAPER_MODULE_NAME);
-    if (!subKey) return false;
-    if (ffOptionParseModuleArgs(key, subKey, value, &options->moduleArgs))
-        return true;
-
-    return false;
-}
-
 void ffParseWallpaperJsonObject(FFWallpaperOptions* options, yyjson_val* module)
 {
-    yyjson_val *key_, *val;
+    yyjson_val *key, *val;
     size_t idx, max;
-    yyjson_obj_foreach(module, idx, max, key_, val)
+    yyjson_obj_foreach(module, idx, max, key, val)
     {
-        const char* key = yyjson_get_str(key_);
-        if(ffStrEqualsIgnCase(key, "type"))
-            continue;
-
         if (ffJsonConfigParseModuleArgs(key, val, &options->moduleArgs))
             continue;
 
-        ffPrintError(FF_WALLPAPER_MODULE_NAME, 0, &options->moduleArgs, FF_PRINT_TYPE_DEFAULT, "Unknown JSON key %s", key);
+        ffPrintError(FF_WALLPAPER_MODULE_NAME, 0, &options->moduleArgs, FF_PRINT_TYPE_DEFAULT, "Unknown JSON key %s", unsafe_yyjson_get_str(key));
     }
 }
 
 void ffGenerateWallpaperJsonConfig(FFWallpaperOptions* options, yyjson_mut_doc* doc, yyjson_mut_val* module)
 {
-    __attribute__((__cleanup__(ffDestroyWallpaperOptions))) FFWallpaperOptions defaultOptions;
-    ffInitWallpaperOptions(&defaultOptions);
-
-    ffJsonConfigGenerateModuleArgsConfig(doc, module, &defaultOptions.moduleArgs, &options->moduleArgs);
+    ffJsonConfigGenerateModuleArgsConfig(doc, module, &options->moduleArgs);
 }
 
 void ffGenerateWallpaperJsonResult(FF_MAYBE_UNUSED FFWallpaperOptions* options, yyjson_mut_doc* doc, yyjson_mut_val* module)
@@ -89,31 +70,27 @@ void ffGenerateWallpaperJsonResult(FF_MAYBE_UNUSED FFWallpaperOptions* options,
     yyjson_mut_obj_add_strbuf(doc, module, "result", &fullpath);
 }
 
-void ffPrintWallpaperHelpFormat(void)
-{
-    FF_PRINT_MODULE_FORMAT_HELP_CHECKED(FF_WALLPAPER_MODULE_NAME, "{1}", FF_WALLPAPER_NUM_FORMAT_ARGS, ((const char* []) {
-        "File name",
-        "Full path",
-    }));
-}
-
 void ffInitWallpaperOptions(FFWallpaperOptions* options)
 {
-    ffOptionInitModuleBaseInfo(
-        &options->moduleInfo,
-        FF_WALLPAPER_MODULE_NAME,
-        "Print image file path of current wallpaper",
-        ffParseWallpaperCommandOptions,
-        ffParseWallpaperJsonObject,
-        ffPrintWallpaper,
-        ffGenerateWallpaperJsonResult,
-        ffPrintWallpaperHelpFormat,
-        ffGenerateWallpaperJsonConfig
-    );
-    ffOptionInitModuleArg(&options->moduleArgs);
+    ffOptionInitModuleArg(&options->moduleArgs, "󰸉");
 }
 
 void ffDestroyWallpaperOptions(FFWallpaperOptions* options)
 {
     ffOptionDestroyModuleArg(&options->moduleArgs);
 }
+
+FFModuleBaseInfo ffWallpaperModuleInfo = {
+    .name = FF_WALLPAPER_MODULE_NAME,
+    .description = "Print image file path of current wallpaper",
+    .initOptions = (void*) ffInitWallpaperOptions,
+    .destroyOptions = (void*) ffDestroyWallpaperOptions,
+    .parseJsonObject = (void*) ffParseWallpaperJsonObject,
+    .printModule = (void*) ffPrintWallpaper,
+    .generateJsonResult = (void*) ffGenerateWallpaperJsonResult,
+    .generateJsonConfig = (void*) ffGenerateWallpaperJsonConfig,
+    .formatArgs = FF_FORMAT_ARG_LIST(((FFModuleFormatArg[]) {
+        {"File name", "file-name"},
+        {"Full path", "full-path"},
+    }))
+};
