@@ -1,9 +1,11 @@
 #pragma once
 
-#include "fastfetch.h"
+#include "option.h"
 
 #define FF_ZPOOL_MODULE_NAME "Zpool"
 
 void ffPrintZpool(FFZpoolOptions* options);
 void ffInitZpoolOptions(FFZpoolOptions* options);
 void ffDestroyZpoolOptions(FFZpoolOptions* options);
+
+extern FFModuleBaseInfo ffZpoolModuleInfo;
