@@ -9,7 +9,12 @@
 #include <string.h>
 #include <stdlib.h>
 #include <assert.h>
-#include "3rdparty/yyjson/yyjson.h"
+
+#ifdef FF_USE_SYSTEM_YYJSON
+    #include <yyjson.h>
+#else
+    #include "3rdparty/yyjson/yyjson.h"
+#endif
 
 #ifdef _WIN32
     // #include <shlwapi.h>
